/*
 * Copyright (C) 2001 Simon Ekstrand <simon@routemeister.net>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 1, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#ifndef SUB_H
#define SUB_H
#ifdef HAVE_CONFIG_H
#include <config.h>
#endif
#ifdef HAVE_SYS_TYPES_H
#include <sys/types.h>
#endif
#ifdef HAVE_SYS_BITYPES_H
#include <sys/bitypes.h>
#endif
#include <sys/socket.h>
#include <net/if.h>

#ifdef PACKAGE
#define NAME PACKAGE
#else
#define NAME "sipcalc"
#endif

#if !defined(HAVE_U_INT32_T) && defined(HAVE_UINT32_T)
#define u_int32_t uint32_t
#endif
#if defined(HAVE_U_INT32_T) && !defined(HAVE_UINT32_T)
#define uint32_t u_int32_t
#endif

#define ADDR_VAL "0123456789."

struct addr {
	char class;
	char class_remark[64];
	char pres_bitmap[36];
	int n_nmaskbits;
	u_int32_t n_cbroadcast;
	u_int32_t n_broadcast;
	u_int32_t n_cnaddr;
	u_int32_t n_naddr;
	u_int32_t n_cnmask;
	u_int32_t n_nmask;
	u_int32_t n_haddr;
	u_int32_t i_broadcast;
};

struct if_info {
	char name[IFNAMSIZ+1], p_addr[19], p_nmask[16];
	short flags;
	struct addr ad;
	struct if_info *next;
};

struct misc_args {
	int numnets;
	u_int32_t splitmask;
};

#define CF_INFO     0x01
#define CF_BITMAP   0x02
#define CIDR_INFO   0x04
#define CIDR_BITMAP 0x08
#define NET_INFO    0x10
#define SPLIT       0x20

/*
 * prototypes
 */
int validate_addr(char *in_addr);
int getsplitnum(char *buf, u_int32_t *splitmask);
int quadtonum(char *quad, u_int32_t *num);
char *numtoquad(u_int32_t num);
char *numtobitmap(u_int32_t num);
int get_addr(struct if_info *ifi);

#endif /* SUB_H */

