/*
 * Copyright (C) 2001 Simon Ekstrand <simon@routemeister.net>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 1, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
#ifdef HAVE_CONFIG_H
#include <config.h>
#endif
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/types.h>
#include "sub.h"
#include "sub-o.h"

/*
 * 0 error
 * 1 normal
 */
int validate_addr(char *addr) {
	int x, y, z, m;
	char buf[16];
	char *sl;

	if(strlen(addr) < 7 || strlen(addr) > 15)
		return 0;

	x=0;
	y=0;
	while(x < strlen(addr)) {
		if(addr[x]=='.')
			y++;
		x++;
	}
	if(y!=3)
		return 0;

	x=0;
	y=0;
	while(x < strlen(addr)) {
		z=0;
		y=0;
		while(z < strlen(ADDR_VAL) && !y) {
			if(addr[x]==ADDR_VAL[z])
				y=1;
			z++;
		}
		if(!y)
			return 0;
		x++;
	}

	if(addr[0]=='.' || addr[strlen(addr)-1]=='.')
		return 0;

	x=0;
	while(x < (strlen(addr)-1)) {
		if(addr[x]=='.' && addr[x+1]=='.')
			return 0;
		x++;
	}

	y=0;
	for(x=0; x < 4; x++) {
		z=0;
		bzero((char *) buf, 16);
		while(addr[y]!='.') {
			buf[z]=addr[y];
			y++;
			z++;
		}
		if(z > 3)
			return 0;
		m=atoi(buf);
		if(m < 0 || m > 255)
			return 0;
	};


	return 1;
}

/*
 * 0 error
 * 1 normal
 * 2 /x
 * 3 hex
 */
int validate_netmask(char *in_addr) {
	int x, y, z, m;
	char buf[16];
	char addr[16];
	char *sl;
	u_int32_t sm;

	x=0;
	y=0;
	z=0;
	while(x < strlen(in_addr) && !y) {
		if(!isxdigit(in_addr[x]) && in_addr[x]!='x' && in_addr[x]!='X')
			y=1;
		if(in_addr[x]=='x' || in_addr[x]=='X') {
			z++;
			y=0;
		}
		x++;
	}

	x=0;
	if(!y && z==1) {
		x=0;
		if(in_addr[0]=='x' || in_addr[0]=='X')
			x=1;
		if(!x) {
			if(in_addr[0]=='0' && (in_addr[1]=='x' || in_addr[1]=='X'))
				x=1;
		}
	}
	if(x==1)
		return 3;

	bzero((char *) addr, 16);
	if(strstr(in_addr, "/")) {
		x=0;
		while(in_addr[x]!='/') {
			addr[x]=in_addr[x];
			x++;
		}
	}
	else {
		strncpy(addr, in_addr, 16);
	}

	/*
	 * This also calls validate_addr.
	 */
	if(quadtonum(addr, &sm))
		return 0;

	if(!(sl=strstr(in_addr, "/"))) {
		y=0;
		for(x=0; x < 32; x++) {
			if(!y && !((sm >> (31-x)) & 1))
				y=1;
			if(y==1 && ((sm >> (31-x)) & 1))
				y=2;
		}
		if(y==2)
			return -1;

		/*
		 * valid
		 */
		return 1;
	}

	sl++;
	if(strlen(sl) < 1 || strlen(sl) > 2)
		return 0;

	if(sl[0]=='.' || sl[1]=='.')
		return 0;

	x=0;
	y=0;
	while(x < strlen(sl)) {
		z=0;
		y=0;
		while(z < strlen(ADDR_VAL) && !y) {
			if(sl[x]==ADDR_VAL[z])
				y=1;
			z++;
		}
		if(!y)
			return 0;
		x++;
	}

	m=atoi(sl);
	if(m < 0 || m > 32)
		return 0;

	return 2;
}

int getsplitnum(char *buf, u_int32_t *splitmask) {
	int x, y;
	u_int32_t sm;

	if(strlen(buf) < 1)
		return -1;

	if(strlen(buf) < 4) {
		if(buf[0]=='/')
			buf++;
		y=atoi(buf);
		if(y < 1 || y > 32)
			return -1;
		sm=0;
		for(x=0; x < y; x++)
			sm=sm | (1 << (31-x));
	}
	else {
		if(!strstr(buf, "."))
			sm=strtoul(buf, (char **)NULL, 16);
		else {
			if(quadtonum(buf, &sm))
				return -1;
		}
	}
	y=0;
	for(x=0; x < 32; x++) {
		if(!y && !((sm >> (31-x)) & 1))
			y=1;
		if(y==1 && ((sm >> (31-x)) & 1))
			y=2;
	}
	if(y==2)
		return -1;

	*splitmask=sm;

	return 0;
}

int quadtonum(char *quad, u_int32_t *num) {
	char buf[128];
	int x, y, z;

	if(validate_addr(quad)!=1)
		return -1;

	bzero((char *) buf, 128);
	x=0;
	while(quad[x]!='.') {
		buf[x]=quad[x];
		x++;
	}
	*num=0;
	x++;
	for(y=0; y < 4; y++) {
		z=atoi(buf);
		if(z > 255 || z < 0)
			return -1;
		*num=*num | (z << (8*(3-y)));
		bzero((char *) buf, 128);
		z=0;
		while(quad[x]!='.' && quad[x]!='\0' && x < strlen(quad)) {
			buf[z]=quad[x];
			x++;
			z++;
		}
		x++;
	}

	return 0;
}

char *numtoquad(u_int32_t num) {
	int a[4], x;
	static char quad[17];

	for(x=0; x < 4; x++)
		a[x]=num >> (8*(3-x)) & 0xff;
	bzero((char *) quad, 17);
	snprintf(quad, 16, "%d.%d.%d.%d", a[0], a[1], a[2], a[3]);

	return quad;
}

char *numtobitmap(u_int32_t num) {
	static char bitmap[36];
	int x, y, z;

	bzero((char *) bitmap, 36);
	y=1;
	z=0;
	for(x=0; x < 32; x++) {
		if(!((num >> (31-x)) & 1))
			bitmap[z]='0';
		else
			bitmap[z]='1';
		if(y==8 && z < 34) {
			z++;
			bitmap[z]='.';
			y=0;
		}
		y++;
		z++;
	}

	return bitmap;
}

int parse_addr(struct if_info *ifi) {
	char buf[128], buf2[128];
	char *s_find;
	int x;

	bzero((char *) buf, 128);
	bzero((char *) buf2, 128);
	ifi->ad.n_nmaskbits=0;

	/*
	 * netmask
	 */
	if(ifi->p_nmask[0]=='\0') {
		/*
		 * "/netmask"
		 */
		if(!(s_find=strstr(ifi->p_addr, "/"))) {
			ifi->ad.n_nmask=0xffffffff;
			ifi->ad.n_nmaskbits=32;
		}
		else {
			*s_find='\0';
			s_find++;
			if(!*s_find)
				return -1;
			buf[0]=*s_find;
			*s_find='\0';
			s_find++;
			if(*s_find) {
				buf[1]=*s_find;
				*s_find='\0';
			}
			ifi->ad.n_nmaskbits=atoi(buf);

			ifi->ad.n_nmask=0;
			for(x=0; x < ifi->ad.n_nmaskbits; x++)
				ifi->ad.n_nmask=ifi->ad.n_nmask | (1 << (31-x));
		}
	}
	else {
		/*
		 * hex netmask
		 */
		if(!strstr(ifi->p_nmask, "."))
			ifi->ad.n_nmask=strtoul(ifi->p_nmask, (char **)NULL, 16);
		/*
		 * "normal" netmask
		 */
		else {
			if(quadtonum(ifi->p_nmask, &ifi->ad.n_nmask))
				return -2;
		}
	}

	if(ifi->ad.n_nmaskbits < 0 || ifi->ad.n_nmaskbits > 32)
		return -2;

	/*
	 * host address
	 */
	if(quadtonum(ifi->p_addr, &ifi->ad.n_haddr))
		return -1;

	return 0;
}

/*
 * return
 * 0  ok
 * -1 invalid address
 * -2 invalid netmask
 */
int get_addr(struct if_info *ifi) {
	int x, y;

	x=0;
	if(!ifi->name[0])
		x=parse_addr(ifi);

	if(x)
		return x;

	/*
	 * count netmask bits
	 */
	ifi->ad.n_nmaskbits=0;
	for(x=0; x < 32; x++) {
		if((ifi->ad.n_nmask >> x) & 1)
			ifi->ad.n_nmaskbits++;
	}
	if(ifi->ad.n_nmaskbits < 0 || ifi->ad.n_nmaskbits > 32)
		return -2;

	/*
	 * validate netmask
	 */
	y=0;
	for(x=0; x < 32; x++) {
		if(!y && !((ifi->ad.n_nmask >> (31-x)) & 1))
			y=1;
		if(y==1 && ((ifi->ad.n_nmask >> (31-x)) & 1))
			y=2;
	}
	if(y==2)
		return -2;


	/*
	 * network class, class remark and classfull netmask
	 */
	bzero((char *) ifi->ad.class_remark, 64);
	x=ifi->ad.n_haddr >> 24;
	ifi->ad.n_cnaddr=0;
	if(!(x & 0x80)) {
		ifi->ad.class='A';
		ifi->ad.n_cnmask=0xff000000;
	}
	if((x & 0xc0)==0x80) {
		ifi->ad.class='B';
		ifi->ad.n_cnmask=0xffff0000;
	}
	if((x & 0xe0)==0xc0) {
		ifi->ad.class='C';
		ifi->ad.n_cnmask=0xffffff00;
	}
	if((x & 0xf0)==0xe0) {
		ifi->ad.class='D';
		snprintf(ifi->ad.class_remark, 64, " (multicast network)");
		ifi->ad.n_cnmask=ifi->ad.n_nmask;
	}
	if((x & 0xf8)==0xf0) {
		ifi->ad.class='E';
		snprintf(ifi->ad.class_remark, 64, " (reserved for future use)");
		ifi->ad.n_cnmask=ifi->ad.n_nmask;
	}
	if(ifi->ad.class=='\0') {
		ifi->ad.n_cnmask=ifi->ad.n_nmask;
		snprintf(ifi->ad.class_remark, 64, "Nonexistant");
	}

	/*
	 * network address (classfull + cidr)
	 */
	ifi->ad.n_naddr=ifi->ad.n_haddr & ifi->ad.n_nmask;
	ifi->ad.n_cnaddr=ifi->ad.n_haddr & ifi->ad.n_cnmask;

	/*
	 * cidr broadcast address
	 */
	ifi->ad.n_broadcast=0xffffffff-ifi->ad.n_nmask+ifi->ad.n_naddr;

	return 0;
}

