/*
 * Copyright (C) 2001 Simon Ekstrand <simon@routemeister.net>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 1, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "sub.h"


void show_split_networks(struct if_info *ifi, u_int32_t splitmask) {
	u_int32_t diff, start, end;
	int x;

	printf("[Split network]\n");
	if(splitmask < ifi->ad.n_nmask) {
		printf("Error: oversized splitmask.\n\n");
		return;
	}
	diff=0xffffffff-splitmask+1;
	start=ifi->ad.n_naddr;
	end=ifi->ad.n_naddr+diff-1;

	x=0;
	while(!x) {
		printf("Network			- %-15s - ", numtoquad(start));
		printf("%s\n", numtoquad(end));
		start+=diff;
		if(end==0xffffffff || end >= ifi->ad.n_broadcast)
			x=1;
		end+=diff;
	}

	printf("\n");

	return;
}

int show_networks(struct if_info *ifi, int count) {
	u_int32_t diff, start, end;
	int x;

	printf("[Networks]\n");
	diff=0xffffffff-ifi->ad.n_nmask+1;
	if(ifi->ad.n_nmask > 0xffffff00 && count==-1) {
		start=ifi->ad.n_naddr & 0xffffff00;
		end=(ifi->ad.n_broadcast & 0xffffff00) + diff-1;
	}
	else {
		start=ifi->ad.n_naddr;
		end=ifi->ad.n_broadcast;
	}

	x=0;
	while(!x && count) {
		printf("Network			- %-15s - ", numtoquad(start));
		printf("%s", numtoquad(end));
		if(start==ifi->ad.n_naddr)
			printf(" (current)\n");
		else
			printf("\n");
		start+=diff;
		if(end==0xffffffff)
			x=1;
		if((end & 0x000000ff) == 0xff && count==-1)
			x=1;
		end+=diff;
		if(count > 0)
			count--;
	}

	printf("\n");

	return 0;
}

void print_cf_info(struct if_info *ifi) {
	printf("[Classfull]\n");
	printf("Host address		- %s\n", numtoquad(ifi->ad.n_haddr));
	printf("Host address (decimal)	- %u\n", ifi->ad.n_haddr);
	printf("Host address (hex)	- %X\n", ifi->ad.n_haddr);
	printf("Network address		- %s\n", numtoquad(ifi->ad.n_cnaddr));
	printf("Network class		- %c%s\n", ifi->ad.class, ifi->ad.class_remark);
	printf("Network mask		- %s\n", numtoquad(ifi->ad.n_cnmask));
	printf("Network mask (hex)	- %X\n", ifi->ad.n_cnmask);
	printf("Broadcast address	- %s\n", numtoquad(ifi->ad.n_cnaddr+(0xffffffff-ifi->ad.n_cnmask)));
	printf("\n");

	return;
}

void print_cf_bitmap(struct if_info *ifi) {
	printf("[Classfull bitmaps]\n");
	printf("Network address		- %s\n", numtobitmap(ifi->ad.n_cnaddr));
	printf("Network mask		- %s\n", numtobitmap(ifi->ad.n_cnmask));
	printf("\n");

	return;
}

void print_cidr_info(struct if_info *ifi) {
	printf("[CIDR]\n");
	printf("Host address		- %s\n", numtoquad(ifi->ad.n_haddr));
	printf("Host address (decimal)	- %u\n", ifi->ad.n_haddr);
	printf("Host address (hex)	- %X\n", ifi->ad.n_haddr);
	printf("Network address		- %s\n", numtoquad(ifi->ad.n_naddr));
	printf("Network mask		- %s\n", numtoquad(ifi->ad.n_nmask));
	printf("Network mask (bits)	- %d\n", ifi->ad.n_nmaskbits);
	printf("Network mask (hex)	- %X\n", ifi->ad.n_nmask);
	printf("Broadcast address	- %s\n", numtoquad(ifi->ad.n_broadcast));
	printf("Cisco wildcard		- %s\n", numtoquad(ifi->ad.n_nmask ^ 0xffffffff));
	if(!ifi->ad.n_nmask)
		printf("Addresses in network	- %u\n", (ifi->ad.n_broadcast)-(ifi->ad.n_naddr));
	else
		printf("Addresses in network	- %u\n", (ifi->ad.n_broadcast)-(ifi->ad.n_naddr)+1);
	printf("Network range		- %s-", numtoquad(ifi->ad.n_naddr));
	printf("%s\n", numtoquad(ifi->ad.n_broadcast));
	if(ifi->ad.n_naddr+1 <= ifi->ad.n_broadcast-1) {
		printf("Usable range		- %s-", numtoquad(ifi->ad.n_naddr+1));
		printf("%s\n", numtoquad(ifi->ad.n_broadcast-1));
	}
	printf("\n");

	return;
}

void print_cidr_bitmap(struct if_info *ifi) {
	printf("[CIDR bitmaps]\n");
	printf("Host address		- %s\n", numtobitmap(ifi->ad.n_haddr));
	printf("Network address		- %s\n", numtobitmap(ifi->ad.n_naddr));
	printf("Network mask		- %s\n", numtobitmap(ifi->ad.n_nmask));
	printf("Broadcast address	- %s\n", numtobitmap(ifi->ad.n_broadcast));
	printf("Cisco wildcard		- %s\n", numtobitmap(ifi->ad.n_nmask ^ 0xffffffff));
	printf("Network range		- %s -\n", numtobitmap(ifi->ad.n_naddr));
	printf("			  %s\n", numtobitmap(ifi->ad.n_broadcast));
	if(ifi->ad.n_naddr+1 <= ifi->ad.n_broadcast-1) {
		printf("Usable range		- %s -\n", numtobitmap(ifi->ad.n_naddr+1));
		printf("			  %s\n", numtobitmap(ifi->ad.n_broadcast-1));
	}
	printf("\n");

	return;
}

#ifdef HAVE_GETOPT_LONG
void print_help() {
	printf("%s %s\n\n", NAME, VERSION);
	printf("Usage: %s [OPTIONS]... <[ADDRESS]... [INTERFACE]... | [-]>\n\n", NAME);
	printf("  -a, --all\t\t\tAll possible information.\n");
	printf("  -b, --cidr-bitmap\t\tCIDR bitmap.\n");
	printf("  -c, --classfull-addr\t\tClassfull address information.\n");
	printf("  -h, --help\t\t\tDisplay this help.\n");
	printf("  -i, --cidr-addr\t\tCIDR address information. (default)\n");
	printf("  -n, --subnets=NUM\t\tDisplay NUM extra subnets (starting from\n");
	printf("\t\t\t\tthe current subnet). Will display all subnets\n");
	printf("\t\t\t\tin the current /24 if NUM is 0.\n");
	printf("  -s, --split=MASK\t\tSplit the current network into subnets\n");
	printf("\t\t\t\tof MASK size.\n");
	printf("  -v, --version\t\t\tVersion information.\n");
	printf("  -x, --classfull-bitmap\tClassfull bitmap.\n");
	printf("\n");
	printf("Address must be in the \"standard\" dotted quad format.\n");
	printf("Netmask can be given in three different ways:\n");
	printf(" - Number of bits    [/nn]\n");
	printf(" - Dotted quad       [nnn.nnn.nnn.nnn]\n");
	printf(" - Hex               [0xnnnnnnnn | nnnnnnnn]\n");
	printf("\n");
	printf("Interface must be a valid network interface on the system.\n");
	printf("If this options is used an attempt will be made to gain the address\n");
	printf("and netmask from the specified interface.\n");
	printf("\n");
	printf("Replacing address/interface with '-' will use stdin for reading further\n");
	printf("arguments.\n");
	printf("\n");
	printf("Report bugs to <simon@routemeister.net>.\n");

	return;
}
#else /* ! HAVE_GETOPT_LONG */
void print_help() {
	printf("%s %s\n\n", NAME, VERSION);
	printf("Usage: %s [OPTIONS]... <[ADDRESS]... [INTERFACE]... | [-]>\n\n", NAME);
	printf("  -a\t\tAll possible information.\n");
	printf("  -b\t\tCIDR bitmap.\n");
	printf("  -c\t\tClassfull address information.\n");
	printf("  -h\t\tDisplay this help.\n");
	printf("  -i\t\tCIDR address information. (default)\n");
	printf("  -n n\t\tDisplay n extra subnets (starting from current subnet).\n");
	printf("\t\tWill display all subnets in the current /24 if n is 0.\n");
	printf("  -s\t\tSplit the current network into subnets of MASK size.\n");
	printf("  -v\t\tVersion information.\n");
	printf("  -x\t\tClassfull bitmap.\n");
	printf("\n");
	printf("Address must be in the \"standard\" dotted quad format.\n");
	printf("Netmask can be given in three different ways:\n");
	printf(" - Number of bits    [/nn]\n");
	printf(" - Dotted quad       [nnn.nnn.nnn.nnn]\n");
	printf(" - Hex               [0xnnnnnnnn | nnnnnnnn]\n");
	printf("\n");
	printf("Interface must be a valid network interface on the system.\n");
	printf("If this options is used an attempt will be made to gain the address\n");
	printf("and netmask from the specified interface.\n");
	printf("\n");
	printf("Replacing address/interface with '-' will use stdin for reading further\n");
	printf("arguments.\n");
	printf("\n");
	printf("Report bugs to <simon@routemeister.net>.\n");

	return;
}
#endif /* HAVE_GETOPT_LONG */

void print_short_help() {
	printf("Usage: %s [OPTIONS]... <[ADDRESS]... [INTERFACE]... | [-]>\n", NAME);
	printf("Try '%s -h' for more information.\n", NAME);
}

void print_version() {
	printf("%s %s\n", NAME, VERSION);
	printf("Written by Simon Ekstrand.\n");
	printf("\n");
	printf("Copyright (C) 2001 Simon Ekstrand.\n");
	printf("This is free software; see the source for copying conditions.  There is NO\n");
	printf("warranty; not even for MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.\n");
}

