/*
 * Copyright (C) 2001 Simon Ekstrand <simon@routemeister.net>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 1, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "sub.h"
#include "sub-owww.h"

void show_split_networks(struct if_info *ifi, u_int32_t splitmask) {
	u_int32_t diff, start, end;
	int x;

	printf("<TR><TD><H3>Split Network</H3></TD></TR>\n");
	if(splitmask < ifi->ad.n_nmask) {
		printf("<TR><TD>Error: oversized splitmask.</TD></TR>\n");
		printf("<TR><TD><BR></TD></TR>\n");
		printf("<TR><TD><BR></TD></TR>\n");
		return;
	}
	diff=0xffffffff-splitmask+1;
	start=ifi->ad.n_naddr;
	end=ifi->ad.n_naddr+diff-1;

	x=0;
	while(!x) {
		printf("<TR><TD>Network</TD><TD>%-15s - ", numtoquad(start));
		printf("%s\n</TD></TR>\n", numtoquad(end));
		start+=diff;
		if(end==0xffffffff || end >= ifi->ad.n_broadcast)
			x=1;
		end+=diff;
	}

	printf("<TR><TD><BR></TD></TR>\n");
	printf("<TR><TD><BR></TD></TR>\n");

	return;
}

int show_networks(struct addr ad, int count) {
	u_int32_t diff, start, end;
	int x;

	diff=0xffffffff-ad.n_nmask+1;
	if(ad.n_nmask > 0xffffff00 && count==-1) {
		start=ad.n_naddr & 0xffffff00;
		end=(ad.n_broadcast & 0xffffff00) + diff-1;
	}
	else {
		start=ad.n_naddr;
		end=ad.n_broadcast;
	}
	printf("<TR><TD><H3>Networks</H3></TD></TR>\n");

	x=0;
	while(!x && count) {
		printf("<TR><TD>Network</TD><TD>%15s-", numtoquad(start));
		printf("%s", numtoquad(end));
		if(start==ad.n_naddr)
			printf(" (current)</TD></TR>\n");
		else
			printf("</TD></TR>\n");
		start+=diff;
		if(end==0xffffffff)
			x=1;
		if((end & 0x000000ff) == 0xff && count==-1)
			x=1;
		end+=diff;
		if(count > 0)
			count--;
	}

	printf("<TR><TD><BR></TD></TR>\n");
	printf("<TR><TD><BR></TD></TR>\n");

	return 0;
}

void print_cf_info(struct addr ad) {
	printf("<TR><TD><H3>Classfull</H3></TD></TR>\n");
	printf("<TR><TD>Host address</TD><TD>%s</TD></TR>\n", numtoquad(ad.n_haddr));
	printf("<TR><TD>Host address (decimal)</TD><TD>%u</TD></TR>\n", ad.n_haddr);
	printf("<TR><TD>Host address (hex)</TD><TD>%X</TD></TR>\n", ad.n_haddr);
	printf("<TR><TD>Network address</TD><TD>%s</TD></TR>\n", numtoquad(ad.n_cnaddr));
	printf("<TR><TD>Network class</TD><TD>%c%s</TD></TR>\n", ad.class, ad.class_remark);
	printf("<TR><TD>Network mask</TD><TD>%s</TD></TR>\n", numtoquad(ad.n_cnmask));
	printf("<TR><TD>Network mask (hex)</TD><TD>%X</TD></TR>\n", ad.n_cnmask);
	printf("<TR><TD>Broadcast address</TD><TD>%s</TD></TR>\n", numtoquad(ad.n_cnaddr+(0xffffffff-ad.n_cnmask)));
	printf("<TR><TD><BR></TD></TR>\n");
	printf("<TR><TD><BR></TD></TR>\n");

	return;
}

void print_cf_bitmap(struct addr ad) {
	printf("<TR><TD><H3>Classfull bitmaps</H3></TD></TR>\n");
	printf("<TR><TD>Network address</TD><TD>%s</TD></TR>\n", numtobitmap(ad.n_cnaddr));
	printf("<TR><TD>Network mask</TD><TD>%s</TD></TR>\n", numtobitmap(ad.n_cnmask));
	printf("<TR><TD><BR></TD></TR>\n");
	printf("<TR><TD><BR></TD></TR>\n");

	return;
}

void print_cidr_info(struct addr ad) {
	printf("<TR><TD><H3>CIDR</H3></TD></TR>\n");
	printf("<TR><TD>Host address</TD><TD>%s</TD></TR>\n", numtoquad(ad.n_haddr));
	printf("<TR><TD>Host address (decimal)</TD><TD>%u</TD></TR>\n", ad.n_haddr);
	printf("<TR><TD>Host address (hex)</TD><TD>%X</TD></TR>\n", ad.n_haddr);
	printf("<TR><TD>Network address</TD><TD>%s</TD></TR>\n", numtoquad(ad.n_naddr));
	printf("<TR><TD>Network mask</TD><TD>%s</TD></TR>\n", numtoquad(ad.n_nmask));
	printf("<TR><TD>Network mask (bits)</TD><TD>%d</TD></TR>\n", ad.n_nmaskbits);
	printf("<TR><TD>Network mask (hex)</TD><TD>%X</TD></TR>\n", ad.n_nmask);
	printf("<TR><TD>Broadcast address</TD><TD>%s</TD></TR>\n", numtoquad(ad.n_broadcast));
	printf("<TR><TD>Cisco wildcard</TD><TD>%s</TD></TR>\n", numtoquad(ad.n_nmask ^ 0xffffffff));
	if(ad.n_naddr+1 <= ad.n_broadcast-1)
		printf("<TR><TD>Hosts/range</TD><TD>%u</TD></TR>\n", (ad.n_broadcast)-(ad.n_naddr+1));
	printf("<TR><TD>Network range</TD><TD>%s-", numtoquad(ad.n_naddr));
	printf("%s</TD></TR>\n", numtoquad(ad.n_broadcast));
	if(ad.n_naddr+1 <= ad.n_broadcast-1) {
		printf("<TR><TD>Usable range</TD><TD>%s-", numtoquad(ad.n_naddr+1));
		printf("%s</TD></TR>\n", numtoquad(ad.n_broadcast-1));
	}
	printf("<TR><TD><BR></TD></TR>\n");
	printf("<TR><TD><BR></TD></TR>\n");

	return;
}

void print_cidr_bitmap(struct addr ad) {
	printf("<TR><TD><H3>CIDR bitmaps</H3></TD></TR>\n");
	printf("<TR><TD>Host address</TD><TD>%s</TD></TR>\n", numtobitmap(ad.n_haddr));
	printf("<TR><TD>Network address</TD><TD>%s</TD></TR>\n", numtobitmap(ad.n_naddr));
	printf("<TR><TD>Network mask</TD><TD>%s</TD></TR>\n", numtobitmap(ad.n_nmask));
	printf("<TR><TD>Broadcast address</TD><TD>%s</TD></TR>\n", numtobitmap(ad.n_broadcast));
	printf("<TR><TD>Cisco wildcard</TD><TD>%s</TD></TR>\n", numtobitmap(ad.n_nmask ^ 0xffffffff));
	printf("<TR><TD>Network range</TD><TD>%s</TD></TR>\n", numtobitmap(ad.n_naddr));
	printf("<TR><TD> </TD><TD>%s</TD></TR>\n", numtobitmap(ad.n_broadcast));
	if(ad.n_naddr+1 <= ad.n_broadcast-1) {
		printf("<TR><TD>Usable range</TD><TD>%s</TD></TR>\n", numtobitmap(ad.n_naddr+1));
		printf("<TR><TD> </TD><TD>%s</TD></TR>\n", numtobitmap(ad.n_broadcast-1));
	}
	printf("<TR><TD><BR></TD></TR>\n");
	printf("<TR><TD><BR></TD></TR>\n");

	return;
}

void print_help(char *name) {
	printf("<DL><DT>\n");
	printf("<H3>For more information about sipcalc see http://www.routemeister.net/<BR></H3>\n");

	return;
}

void print_html_start() {
	printf("Content-type: text/html\n\n");
	printf("<html>\n");
	printf("<head>\n");
	printf("<title>sipcalc-www</title>\n");
	printf("</head>\n");
	printf("<body bgcolor=\"black\" text=#00ff00>\n");

	return;
}

void print_html_end() {
	printf("</body>\n");
	printf("</html>\n");

	return;
}

