/*
 * Copyright (C) 2001 Simon Ekstrand <simon@routemeister.net>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 1, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/types.h>
#include <sys/uio.h>
#include <unistd.h>
#include "sub.h"
#include "sub-owww.h"

char *getarg(char *arg, char *buf) {
	static char retbuf[1024], tmpbuf[1024], ebuf[3];
	char *strfind;
	int x, y, z, error;

	bzero((char *) retbuf, 1024);
	bzero((char *) tmpbuf, 1024);

	snprintf(tmpbuf, 1022, "%s=", arg);
	strfind=strstr(buf, tmpbuf);
	if(!strfind)
		return NULL;

	x=0;
	if(strlen(strfind)==strlen(buf))
		x=1;
	if(!x) {
		strfind--;
		if(strfind[0]=='&')
			x=1;
		strfind++;
	}

	if(!x)
		return NULL;

	strfind+=strlen(arg)+1;
	bzero((char *) tmpbuf, 1024);
	x=0;
	while(x < strlen(strfind) && strfind[x]!='&' && x < 1023) {
		tmpbuf[x]=strfind[x];
		x++;
	}

	y=0;
	error=0;
	for(x=0; x < strlen(tmpbuf); x++) {
		retbuf[y]=tmpbuf[x];
		if(tmpbuf[x]=='+')
			retbuf[y]=' ';
		if(tmpbuf[x]=='%') {
			if(x+2 >= strlen(tmpbuf)) {
				error=1;
				x=strlen(tmpbuf);
			}
			else {
				ebuf[0]=tmpbuf[x+1];
				ebuf[1]=tmpbuf[x+2];
				ebuf[2]='\0';
				z=(int) strtol(ebuf, (char **)NULL, 16);
				retbuf[y]=z;
				x+=2;
			}
		}

		y++;
	}

	if(error)
		return NULL;

	return retbuf;
}

int main(int argc, char *argv[]) {
	int ret, args, x, error, split_err;
	char readbuf[1024], *retbuf, *retbuf2;
	struct if_info ifi;
	struct misc_args m_arg;

	print_html_start();

	bzero((char *) ifi.name, IFNAMSIZ);
	bzero((char *) ifi.p_addr, 19);
	bzero((char *) ifi.p_nmask, 16);
	bzero((char *) readbuf, 1024);
	read(0, (char *) readbuf, 1023);

	if(!strlen(readbuf)) {
		print_help("sub-www");
		print_html_end();
		return 0;
	}

	error=0;
	retbuf=getarg("addr", readbuf);
	if(!retbuf)
		error=1;
	else
		if(retbuf[0]=='\0')
			error=1;

	if(error) {
		print_help("sub-www");
		print_html_end();
		return 0;
	}

	if(strlen(retbuf) > 31)
		error=1;
	else {
		retbuf2=strstr(retbuf, " ");
		if(!retbuf2) {
			strncpy(ifi.p_addr, retbuf, 18);
		}
		else {
			x=0;
			while(x < strlen(retbuf) && retbuf[x]!=' ' && x < 18) {
				ifi.p_addr[x]=retbuf[x];
				x++;
			}
			if(strlen(retbuf2) >= 7) {
				retbuf2++;
					strncpy(ifi.p_nmask, retbuf2, 15);
				retbuf2--;
			}
			else
				error=1;
		}
	}

	if(error) {
		printf("Problem parsing arguments.<BR>\n");
		print_html_end();
		return -1;
	}


	args=0;
	retbuf=getarg("all", readbuf);
	if(retbuf)
		if(!strncmp(retbuf, "on", 2))
			args=args | CF_INFO | CF_BITMAP | CIDR_INFO | CIDR_BITMAP | NET_INFO;
	retbuf=getarg("cidr_bitmap", readbuf);
	if(retbuf)
		if(!strncmp(retbuf, "on", 2))
			args=args | CIDR_BITMAP;
	retbuf=getarg("class_addr", readbuf);
	if(retbuf)
		if(!strncmp(retbuf, "on", 2))
			args=args | CF_INFO;
	retbuf=getarg("cidr_addr", readbuf);
	if(retbuf)
		if(!strncmp(retbuf, "on", 2))
			args=args | CIDR_INFO;
	retbuf=getarg("class_bitmap", readbuf);
	if(retbuf)
		if(!strncmp(retbuf, "on", 2))
			args=args | CF_BITMAP;
	retbuf=getarg("extra_nets", readbuf);
	if(retbuf)
		if(!strncmp(retbuf, "on", 2))
			args=args | NET_INFO;
	retbuf=getarg("split_net", readbuf);
	if(retbuf)
		if(!strncmp(retbuf, "on", 2)) {
			args=args | SPLIT;
			split_err=0;
		}

	m_arg.numnets=-1;
	if((args & NET_INFO)==NET_INFO) {
		retbuf=getarg("num_net", readbuf);
		if(retbuf)
			m_arg.numnets=atoi(retbuf);
	}
	if(m_arg.numnets < 1)
		m_arg.numnets=-1;

	if((args & SPLIT)==SPLIT) {
		retbuf=getarg("split_nets", readbuf);
		if(retbuf) {
			if(getsplitnum(retbuf, &m_arg.splitmask)) {
				printf("Invalid splitmask.<BR><BR>\n");
				split_err=1;
			}
		}
	}

	ret=get_addr(&ifi);

	if(ret==-1) {
		printf("Invalid address.<BR>\n");
		print_html_end();
		return -1;
	}
	if(ret==-2) {
		printf("Invalid netmask.<BR>\n");
		print_html_end();
		return -1;
	}

	if(!args)
		args=CIDR_INFO;

	printf("<BR>\n");
	printf("<TABLE border=\"0\" width=\"50%%\" cellspacing=\"0\">\n");
	if((args & CF_INFO)==CF_INFO)
		print_cf_info(ifi.ad);
	if((args & CIDR_INFO)==CIDR_INFO)
		print_cidr_info(ifi.ad);
	if((args & CF_BITMAP)==CF_BITMAP)
		print_cf_bitmap(ifi.ad);
	if((args & CIDR_BITMAP)==CIDR_BITMAP)
		print_cidr_bitmap(ifi.ad);
	if((args & NET_INFO)==NET_INFO)
		show_networks(ifi.ad, m_arg.numnets);
	if((args & SPLIT)==SPLIT && !split_err)
		show_split_networks(&ifi, m_arg.splitmask);
	printf("</TABLE>\n");

	print_html_end();

	return 0;
}

