/*
 * Copyright (C) 2001 Simon Ekstrand <simon@routemeister.net>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 1, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif
#include <stdio.h>
#ifdef STDC_HEADERS
#include <stdlib.h>
#endif
#include <string.h>
#include <sys/types.h>
#ifdef HAVE_UNISTD_H
#include <unistd.h>
#endif
#if defined(HAVE_GETOPT_LONG) && defined(HAVE_GETOPT_H)
#define _GNU_SOURCE
#include <getopt.h>
#endif
#include "sub.h"
#include "sub-o.h"

extern char *optarg;
extern int optind, opterr, optopt;

int out_int(struct if_info *if_cur, struct if_info *ifarg_cur, int args, struct misc_args m_arg) {
	int iffound, ret;

	iffound=0;
	do {
		for(;;) {
			if(!if_cur) {
				if(!iffound) {
					printf("[%s] - %d\n", ifarg_cur->name, iffound);
					printf("\nUnable to retrieve interface information.\n\n\n");
				}

				return iffound;
			}
			if(!strcmp(if_cur->name, ifarg_cur->name)) {
				printf("[%s] - %d\n", ifarg_cur->name, iffound);
				break;
			}
			if_cur=if_cur->next;
		}

		ret=get_addr(if_cur);

		if(ret==-1)
			printf("\nInvalid address.\n\n\n");
		if(ret==-2)
			printf("\nInvalid netmask.\n\n\n");

		if(!args)
			args=CIDR_INFO;

		if(!ret) {
			printf("\n");
			if((args & CF_INFO)==CF_INFO)
				print_cf_info(if_cur);
			if((args & CIDR_INFO)==CIDR_INFO)
				print_cidr_info(if_cur);
			if((args & CF_BITMAP)==CF_BITMAP)
				print_cf_bitmap(if_cur);
			if((args & CIDR_BITMAP)==CIDR_BITMAP)
				print_cidr_bitmap(if_cur);
			if((args & NET_INFO)==NET_INFO)
				show_networks(if_cur, m_arg.numnets);
			if((args & SPLIT)==SPLIT)
				show_split_networks(if_cur, m_arg.splitmask);
		}

		if(if_cur)
			if_cur=if_cur->next;

		printf("\n");
		iffound++;
	} while(if_cur);

	return iffound;
};

int out_cmdline(struct if_info *ifarg_cur, int args, struct misc_args m_arg) {
	int ret;

	printf("[%s", ifarg_cur->p_addr);
	if(ifarg_cur->p_nmask[0])
		printf(" %s] - 0\n", ifarg_cur->p_nmask);
	else
		printf("] - 0\n");

	ret=get_addr(ifarg_cur);

	if(ret==-1) {
		printf("\nInvalid address.\n\n\n");
		return 0;
	}
	if(ret==-2) {
		printf("\nInvalid netmask.\n\n\n");
		return 0;
	}

	if(!args)
		args=CIDR_INFO;

	printf("\n");
	if((args & CF_INFO)==CF_INFO)
		print_cf_info(ifarg_cur);
	if((args & CIDR_INFO)==CIDR_INFO)
		print_cidr_info(ifarg_cur);
	if((args & CF_BITMAP)==CF_BITMAP)
		print_cf_bitmap(ifarg_cur);
	if((args & CIDR_BITMAP)==CIDR_BITMAP)
		print_cidr_bitmap(ifarg_cur);
	if((args & NET_INFO)==NET_INFO)
		show_networks(ifarg_cur, m_arg.numnets);
	if((args & SPLIT)==SPLIT)
		show_split_networks(ifarg_cur, m_arg.splitmask);
	printf("\n");

	return 0;
};

int cleanline(char *sbuf, char *dbuf) {
	int x, y;

	for(x=0; x < strlen(sbuf); x++) {
		if(sbuf[x]=='\n')
			sbuf[x]=' ';
		if(sbuf[x]=='\t')
			sbuf[x]=' ';
		if(sbuf[x]=='#')
			sbuf[x]='\0';
	}
	x=strlen(sbuf)-1;
	while(sbuf[x]==' ' && x > -1) {
		sbuf[x]='\0';
		x--;
	}
	if(!strlen(sbuf))
		return 0;

	x=0;
	y=0;
	while(x < strlen(sbuf)) {
		if(sbuf[x]==' ' && x) {
			dbuf[y]=' ';
			y++;
		}
		while(sbuf[x]==' ' && x < strlen(sbuf))
			x++;
		while(sbuf[x]!=' ' && x < strlen(sbuf)) {
			dbuf[y]=sbuf[x];
			y++;
			x++;
		}
		if(dbuf[y-1]==' ')
			return 0;
	}
	if(dbuf[strlen(dbuf)-1]==' ')
		dbuf[strlen(dbuf)-1]='\0';

	y=1;
	for(x=0; x < strlen(dbuf); x++)
		if(dbuf[x]==' ')
			y++;

	return y;
}

int get_stdin(struct if_info *ifi) {
	char buf[2], sbuf[128], dbuf[128], *arg1, *arg2;
	int x, y, z, argmax;

	bzero((char *) buf, 2);

	argmax=(IFNAMSIZ+1 > 19) ? IFNAMSIZ+1 : 19;
	arg1=(char *) malloc(argmax);
	arg2=(char *) malloc(16);
	bzero((char *) arg1, argmax);
	bzero((char *) arg2, 16);
	bzero((char *) sbuf, 128);
	bzero((char *) dbuf, 128);

	while(!sbuf[0]) {
		x=0;
		y=0;
		bzero((char *) sbuf, 128);
		do {
			x=read(0, buf, 1);
			if(x==1)
				sbuf[y]=buf[0];
			y++;
		} while(x > 0 && buf[0]!='\n' && y < 127);
		if(x < 0) {
			free(arg1);
			free(arg2);
			return -1;
		}
		if(!x)
			break;

		while(buf[0]!='\n' && x==1)
			x=read(0, buf, 1);
		if(x < 0) {
			free(arg1);
			free(arg2);
			return -1;
		}
		if(!x)
			break;
	};
	if(!sbuf[0]) {
		free(arg1);
		free(arg2);
		return -2;
	}

	x=cleanline(sbuf, dbuf);
	if(x < 1) {
		free(arg1);
		free(arg2);
		return x;
	}

	y=0;
	while(y < strlen(dbuf) && y < argmax && dbuf[y]!=' ') {
		arg1[y]=dbuf[y];
		y++;
	}
	y++;
	z=0;
	while(y < strlen(dbuf) && z < 15 && dbuf[y]!=' ') {
		arg2[z]=dbuf[y];
		y++;
		z++;
	}

	if(x==1) {
		if(strstr(arg1, "/"))
			strncpy(ifi->p_addr, arg1, 18);
		else
			strncpy(ifi->name, arg1, IFNAMSIZ);
	}
	if(x > 1) {
		if(strstr(arg1, "/"))
			strncpy(ifi->p_addr, arg1, 18);
		else {
			strncpy(ifi->p_addr, arg1, 18);
			strncpy(ifi->p_nmask, arg2, 15);
		}
	}

	free(arg1);
	free(arg2);

	return x;
}


int main(int argc, char *argv[]) {
	int x, y, z, args, iffound, error, argcount;
	struct if_info *if_start, *if_cur;
	struct if_info *ifarg_start, *ifarg_cur;
	int ch, parse_stdin;
	struct misc_args m_arg;
	int split_err;
#ifdef HAVE_GETOPT_LONG
	static struct option l_o[] = {
		{"all", no_argument, 0, 'a'},
		{"cidr-bitmap", no_argument, 0, 'b'},
		{"classfull-addr", no_argument, 0, 'c'},
		{"help", no_argument, 0, 'h'},
		{"cidr-addr", no_argument, 0, 'i'},
		{"subnets", required_argument, 0, 'n'},
		{"split", required_argument, 0, 's'},
		{"version", no_argument, 0, 'v'},
		{"classfull-bitmap", no_argument, 0, 'x'},
		{0, 0, 0, 0}
	};
#endif
	if(argc < 2) {
		print_short_help();
		return 0;
	}

	parse_stdin=0;
	args=0;
	m_arg.splitmask=0;
	m_arg.numnets=0;
	split_err=0;
#ifdef HAVE_GETOPT_LONG
	while((ch=getopt_long(argc, argv, "abchHin:s:vVx", l_o, NULL))!=-1) {
#else
	while((ch=getopt(argc, argv, "abchHin:s:vVx"))!=-1) {
#endif
		switch(ch) {
		case 'a' :
			args=args | CF_INFO | CF_BITMAP | CIDR_INFO | CIDR_BITMAP | NET_INFO;
			break;
		case 'b' :
			args=args | CIDR_BITMAP;
			break;
		case 'c' :
			args=args | CF_INFO;
			break;
		case 'h' :
		case 'H' :
			print_help();
			return 0;
		case 'i' :
			args=args | CIDR_INFO;
			break;
		case 'n' :
			args=args | NET_INFO;
			m_arg.numnets=atoi(optarg);
			break;
		case 's' :
			y=getsplitnum(optarg, &m_arg.splitmask);
			if(!y) {
				args=args | SPLIT;
			}
			else {
				printf("Invalid splitmask, unable to split.\n\n");
				split_err=1;
			}
			break;
		case 'v' :
		case 'V' :
			print_version();
			return 0;
		case 'x' :
			args=args | CF_BITMAP;
			break;
		case '?' :
			printf("Try '%s -h' for more information.\n", NAME);
			return 0;
		default :
			print_short_help();
			return 0;
		}
	}

	if(!args && split_err) {
		printf("No valid commands recieved.\n");
		return -1;
	}

	argcount=optind;
	if(!args)
		args=CIDR_INFO;
	if(m_arg.numnets < 1)
		m_arg.numnets=-1;

	if(argv[argcount]) {
		if(argv[argcount][0]=='-' && argv[argcount][1]=='\0')
			parse_stdin=1;
	}
	else {
		print_short_help();
		return 0;
	}

	ifarg_cur=ifarg_start=(struct if_info *) malloc(sizeof(struct if_info));
	ifarg_cur->next=NULL;
	bzero((char *) ifarg_cur->name, IFNAMSIZ);
	bzero((char *) ifarg_cur->p_addr, 19);
	bzero((char *) ifarg_cur->p_nmask, 16);

	iffound=0;
	error=0;
	while(argv[argcount] && !error && !parse_stdin) {
		x=validate_addr(argv[argcount]);
		if(!x)
			x=validate_netmask(argv[argcount]);
		y=-1;
		if(argcount+1 < argc)
				y=validate_netmask(argv[argcount+1]);

		if(!x) {
			if(strlen(argv[argcount]) <= IFNAMSIZ) {
				strncpy(ifarg_cur->name, argv[argcount], IFNAMSIZ);
				iffound=1;
			}
			else
				error=1;
		}

		if(x)
			strncpy(ifarg_cur->p_addr, argv[argcount], 18);

		if(x && (y==1 || y==3)) {
			strncpy(ifarg_cur->p_nmask, argv[argcount+1], 15);
			argcount++;
		}

		argcount++;
		if(argv[argcount] && !error) {
			ifarg_cur->next=(struct if_info *) malloc(sizeof(struct if_info));
			ifarg_cur=ifarg_cur->next;
			ifarg_cur->next=NULL;
			bzero((char *) ifarg_cur->name, IFNAMSIZ+1);
			bzero((char *) ifarg_cur->p_addr, 19);
			bzero((char *) ifarg_cur->p_nmask, 16);
		}
	}
	if(error) {
		free_if(ifarg_start);
		printf("Problem parsing arguments.\n");
		return -1;
	}

	if_start=NULL;
	if_cur=NULL;
	if(iffound || parse_stdin) {
		if(!(if_cur=if_start=get_if_ext())) {
			printf("Unable to retrieve interface information.\n");
			printf("Will only parse none interface arguments.\n\n");
		}
	}

	iffound=0;
	ifarg_cur=ifarg_start;
	while(ifarg_cur && !parse_stdin) {
		if(ifarg_cur->name[0])
			iffound+=out_int(if_cur, ifarg_cur, args, m_arg);
		else
			iffound+=out_cmdline(ifarg_cur, args, m_arg);

		ifarg_cur=ifarg_cur->next;
	}

	z=0;
	y=1;
	while(parse_stdin && y > -1) {
		bzero((char *) ifarg_cur->name, IFNAMSIZ);
		bzero((char *) ifarg_cur->p_addr, 19);
		bzero((char *) ifarg_cur->p_nmask, 16);
		y=get_stdin(ifarg_cur);
		if(y > 0) {
			if(ifarg_cur->name[0])
				iffound+=out_int(if_cur, ifarg_cur, args, m_arg);
			else
				iffound+=out_cmdline(ifarg_cur, args, m_arg);
			z=1;
		}
		if(y==-1)
			printf("\nProblem parsing stdin\n\n");
	}

	if(!z && parse_stdin)
		printf("No arguments found on stdin.\n\n");

	free_if(if_start);
	free_if(ifarg_start);

	return iffound;
}

